@@ -51,6 +51,7 @@
 #include "gpm-debug.h"
 #include "gpm-dpms.h"
 #include "gpm-idle.h"
+#include "gpm-info.h"
 #include "gpm-power.h"
 #include "gpm-hal-monitor.h"
 #include "gpm-brightness.h"
@@ -59,8 +60,8 @@
 #include "gpm-manager.h"
 
 static void     gpm_manager_class_init (GpmManagerClass *klass);
-static void     gpm_manager_init       (GpmManager      *manager);
-static void     gpm_manager_finalize   (GObject         *object);
+static void     gpm_manager_init	(GpmManager      *manager);
+static void     gpm_manager_finalize   (GObject	  *object);
 
 #define GPM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_MANAGER, GpmManagerPrivate))
 
@@ -73,12 +74,29 @@ typedef enum {
 	GPM_WARNING_ACTION = 5
 } GpmWarning;
 
+#define GPM_BUTTON_POWER		"power"
+#define GPM_BUTTON_SLEEP		"sleep"
+#define GPM_BUTTON_SUSPEND		"suspend"
+#define GPM_BUTTON_HIBERNATE		"hibernate"
+#define GPM_BUTTON_LID			"lid"
+#define GPM_BUTTON_BRIGHT_UP		"brightness-up"
+#define GPM_BUTTON_BRIGHT_DOWN		"brightness-down"
+#define GPM_BUTTON_BRIGHT_UP_DEP	"brightness_up"		/* Remove when we depend on HAL 0.5.8 */
+#define GPM_BUTTON_BRIGHT_DOWN_DEP	"brightness_down"	/* as these are the old names */
+#define GPM_BUTTON_LOCK			"lock"
+/* Using www until we get a better one defined for us by the kernel */
+#define GPM_BUTTON_BATTERY		"www"
+
+#define GPM_NOTIFY_TIMEOUT_LONG		20	/* seconds */
+#define GPM_NOTIFY_TIMEOUT_SHORT	5	/* seconds */
+
 struct GpmManagerPrivate
 {
 	GConfClient	*gconf_client;
 
 	GpmDpms		*dpms;
 	GpmIdle		*idle;
+	GpmInfo		*info;
 	GpmPower	*power;
 	GpmBrightness   *brightness;
 
@@ -95,6 +113,10 @@ struct GpmManagerPrivate
 	gboolean	 use_time_to_notify;
 	gboolean	 lid_is_closed;
 
+	time_t           last_resume_event;
+	int		 suppress_policy_timeout;
+
+	const char	*reason;
 };
 
 enum {
@@ -111,11 +133,12 @@ static GObjectClass *parent_class = NULL;
 static guint	     signals [LAST_SIGNAL] = { 0, };
 
 static GConfEnumStringPair icon_policy_enum_map [] = {
-       { GPM_ICON_POLICY_ALWAYS,       "always"   },
-       { GPM_ICON_POLICY_CHARGE,       "charge"   },
-       { GPM_ICON_POLICY_CRITICAL,     "critical" },
-       { GPM_ICON_POLICY_NEVER,        "never"    },
-       { 0, NULL }
+	{ GPM_ICON_POLICY_ALWAYS,	"always"   },
+	{ GPM_ICON_POLICY_PRESENT,	"present"  },
+	{ GPM_ICON_POLICY_CHARGE,	"charge"   },
+	{ GPM_ICON_POLICY_CRITICAL,     "critical" },
+	{ GPM_ICON_POLICY_NEVER,	"never"    },
+	{ 0, NULL }
 };
 
 G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
@@ -135,6 +158,11 @@ G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
 #undef DISABLE_ACTIONS_FOR_TESTING
 /*#define DISABLE_ACTIONS_FOR_TESTING 1*/
 
+/* prototypes */
+static gboolean gpm_manager_suspend (GpmManager *manager, GError **error);
+static gboolean gpm_manager_hibernate (GpmManager *manager, GError **error);
+static gboolean gpm_manager_shutdown (GpmManager *manager, GError **error);
+
 GQuark
 gpm_manager_error_quark (void)
 {
@@ -147,6 +175,19 @@ gpm_manager_error_quark (void)
 	return quark;
 }
 
+/* Returns if an action is valid, i.e. if the difference in time between this
+   request for an action, and the last action completing is larger than the
+   timeout set in gconf. This should fix lots of ACPI bugs we are having. */
+static gboolean
+gpm_manager_is_policy_timout_valid (GpmManager *manager)
+{
+	if ((time (NULL) - manager->priv->last_resume_event) <=
+	    manager->priv->suppress_policy_timeout) {
+		return FALSE;
+	}
+	return TRUE;
+}
+
 gboolean
 gpm_manager_can_suspend (GpmManager *manager,
 			 gboolean   *can,
@@ -252,13 +293,9 @@ get_stock_id_helper (GpmPowerBatteryStatus *device_status, const char *prefix)
 	char *index;
 	char *filename = NULL;
 
-	if (!device_status->is_charging &&
-	    !device_status->is_discharging &&
-	    device_status->percentage_charge > 90) {
+	if (gpm_power_battery_is_charged (device_status)) {
 
-		/* We have to do the additional check for 90% as
-		   some batteries are broken */
-		filename = g_strdup ("battery-charged");
+		filename = g_strdup_printf ("%s-charged", prefix);
 
 	} else if (device_status->is_charging) {
 
@@ -269,6 +306,7 @@ get_stock_id_helper (GpmPowerBatteryStatus *device_status, const char *prefix)
 
 		index = get_icon_index_from_percent (device_status->percentage_charge);
 		filename = g_strdup_printf ("%s-discharging-%s", prefix, index);
+
 	} else {
 
 		/* We have a broken battery, not sure what to display here */
@@ -282,7 +320,7 @@ get_stock_id_helper (GpmPowerBatteryStatus *device_status, const char *prefix)
 /* must free retval */
 static char *
 get_stock_id (GpmManager *manager,
-	      int         icon_policy)
+	      int	  icon_policy)
 {
 	GpmPowerBatteryStatus status_primary;
 	GpmPowerBatteryStatus status_ups;
@@ -290,7 +328,6 @@ get_stock_id (GpmManager *manager,
 	GpmPowerBatteryStatus status_keyboard;
 	gboolean on_ac;
 	gboolean present;
-	gboolean use_fallback;
 
 	gpm_debug ("Getting stock icon for tray");
 
@@ -371,19 +408,15 @@ get_stock_id (GpmManager *manager,
 		return get_stock_id_helper (&status_ups, ICON_PREFIX_UPS);
 	}
 
-	/* we have a falback here, so that desktops by default do not have
-	   the icon, but it can be "forced" */
-	use_fallback = gconf_client_get_bool (manager->priv->gconf_client,
-					      GPM_PREF_USE_ICON_DESKTOPS,
-					      NULL);
-	if (use_fallback) {
-		/* we fallback to the ac_adapter icon */
-		gpm_debug ("Using fallback");
-		return g_strdup_printf (GPM_STOCK_AC_ADAPTER);
+	/* Check if we should just fallback to the ac icon */
+	if (icon_policy == GPM_ICON_POLICY_PRESENT) {
+		gpm_debug ("no devices present, so no icon will be displayed.");
+		return NULL;
 	}
-	/* no icon! */
-	gpm_debug ("No devices present, and no fallback, so no icon displayed.");
-	return NULL;
+
+	/* we fallback to the ac_adapter icon */
+	gpm_debug ("Using fallback");
+	return g_strdup_printf (GPM_STOCK_AC_ADAPTER);
 }
 
 static void
@@ -624,6 +657,11 @@ manager_policy_do (GpmManager *manager,
 		return;
 	}
 
+	if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+		gpm_debug ("Skipping suppressed policy event");
+		return;
+	}
+
 	if (strcmp (action, ACTION_NOTHING) == 0) {
 
 		gpm_debug ("*ACTION* Doing nothing");
@@ -648,6 +686,14 @@ manager_policy_do (GpmManager *manager,
 		gpm_debug ("*ACTION* Shutdown");
 		gpm_manager_shutdown (manager, NULL);
 
+	} else if (strcmp (action, ACTION_INTERACTIVE) == 0) {
+
+		gpm_debug ("*ACTION* Interactive");
+		/* Log out interactively */
+		gnome_client_request_save (gnome_master_client (),
+					   GNOME_SAVE_GLOBAL,
+					   TRUE, GNOME_INTERACT_ANY, FALSE, TRUE);
+
 	} else {
 		gpm_warning ("unknown action %s", action);
 	}
@@ -657,8 +703,8 @@ manager_policy_do (GpmManager *manager,
 
 gboolean
 gpm_manager_get_on_ac (GpmManager *manager,
-		       gboolean	  *on_ac,
-		       GError    **error)
+			gboolean	  *on_ac,
+			GError    **error)
 {
 	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
 
@@ -709,7 +755,23 @@ gpm_manager_get_dpms_mode (GpmManager  *manager,
 	return ret;
 }
 
-gboolean
+/* we set the reason to be WHY. e.g. "user pressed hibernate button" */
+static void
+gpm_manager_set_reason (GpmManager *manager,
+			const char *reason)
+{
+	manager->priv->reason = reason;
+}
+
+/* we set the reason to be WHAT. e.g. "Hibernate system" */
+static void
+gpm_manager_log_reason (GpmManager *manager,
+			const char *action)
+{
+	gpm_syslog ("%s because %s", action, manager->priv->reason);
+}
+
+static gboolean
 gpm_manager_shutdown (GpmManager *manager,
 		      GError    **error)
 {
@@ -730,17 +792,17 @@ gpm_manager_shutdown (GpmManager *manager,
 				   GNOME_SAVE_GLOBAL,
 				   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
 
-	/* FIXME: make this return success/fail */
 	/* FIXME: make this async? */
+	gpm_manager_log_reason (manager, "Shutting down computer");
 	gpm_hal_shutdown ();
 	ret = TRUE;
 
 	return ret;
 }
 
-gboolean
+static gboolean
 gpm_manager_hibernate (GpmManager *manager,
-		       GError    **error)
+			GError    **error)
 {
 	gboolean allowed;
 	gboolean ret;
@@ -766,6 +828,7 @@ gpm_manager_hibernate (GpmManager *manager,
 	gpm_networkmanager_sleep ();
 
 	/* FIXME: make this async? */
+	gpm_manager_log_reason (manager, "Hibernating computer");
 	ret = gpm_hal_hibernate ();
 	if (! ret) {
 		char *message;
@@ -778,7 +841,7 @@ gpm_manager_hibernate (GpmManager *manager,
 						     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
 						     _("hibernate"), GPM_FAQ_URL);
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-					      5000,
+					      GPM_NOTIFY_TIMEOUT_LONG,
 					      _("Hibernate Problem"),
 					      NULL,
 					      message);
@@ -791,10 +854,13 @@ gpm_manager_hibernate (GpmManager *manager,
 	}
 	gpm_networkmanager_wake ();
 
+	/* save the time that we resumed */
+	manager->priv->last_resume_event = time (NULL);
+
 	return ret;
 }
 
-gboolean
+static gboolean
 gpm_manager_suspend (GpmManager *manager,
 		     GError    **error)
 {
@@ -822,6 +888,7 @@ gpm_manager_suspend (GpmManager *manager,
 	gpm_networkmanager_sleep ();
 
 	/* FIXME: make this async? */
+	gpm_manager_log_reason (manager, "Suspending computer");
 	ret = gpm_hal_suspend (0);
 	if (! ret) {
 		char *message;
@@ -834,7 +901,7 @@ gpm_manager_suspend (GpmManager *manager,
 						     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
 						     _("suspend"), GPM_FAQ_URL);
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-					      5000,
+					      GPM_NOTIFY_TIMEOUT_LONG,
 					      _("Suspend Problem"),
 					      NULL,
 					      message);
@@ -847,9 +914,39 @@ gpm_manager_suspend (GpmManager *manager,
 	}
 	gpm_networkmanager_wake ();
 
+	/* save the time that we resumed */
+	manager->priv->last_resume_event = time (NULL);
+
 	return ret;
 }
 
+gboolean
+gpm_manager_suspend_dbus_method (GpmManager *manager,
+				 GError    **error)
+{
+	/* FIXME: From where? */
+	gpm_manager_set_reason (manager, "the DBUS method Suspend() was invoked");
+	return gpm_manager_suspend (manager, error);
+}
+
+gboolean
+gpm_manager_hibernate_dbus_method (GpmManager *manager,
+				   GError    **error)
+{
+	/* FIXME: From where? */
+	gpm_manager_set_reason (manager, "the DBUS method Hibernate() was invoked");
+	return gpm_manager_hibernate (manager, error);
+}
+
+gboolean
+gpm_manager_shutdown_dbus_method (GpmManager *manager,
+				  GError    **error)
+{
+	/* FIXME: From where? */
+	gpm_manager_set_reason (manager, "the DBUS method Shutdown() was invoked");
+	return gpm_manager_shutdown (manager, error);
+}
+
 static void
 idle_changed_cb (GpmIdle    *idle,
 		 GpmIdleMode mode,
@@ -858,6 +955,16 @@ idle_changed_cb (GpmIdle    *idle,
 	GError  *error;
 	gboolean do_laptop_dim;
 
+	/* Ignore timeout events when the lid is closed, as the DPMS is
+	   already off, and we don't want to perform policy actions or re-enable
+	   the screen when the user moves the mouse on systems that do not
+	   support hardware blanking.
+	   Details are here: https://launchpad.net/malone/bugs/22522 */
+	if (manager->priv->lid_is_closed) {
+		gpm_debug ("lid is closed, so we are ignoring idle state changes");
+		return;
+	}
+
 	switch (mode) {
 	case GPM_IDLE_MODE_NORMAL:
 		gpm_debug ("Idle state changed: NORMAL");
@@ -871,7 +978,7 @@ idle_changed_cb (GpmIdle    *idle,
 
 		/* Should we dim the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
-						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
+							GPM_PREF_IDLE_DIM_SCREEN, NULL);
 		if (do_laptop_dim) {
 			/* resume to the previous brightness */
 			gpm_brightness_level_resume (manager->priv->brightness);
@@ -894,7 +1001,7 @@ idle_changed_cb (GpmIdle    *idle,
 
 		/* Should we resume the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
-						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
+							GPM_PREF_IDLE_DIM_SCREEN, NULL);
 		if (do_laptop_dim) {
 			/* save this brightness and dim the screen, fixes #328564 */
 			gpm_brightness_level_save (manager->priv->brightness,
@@ -908,7 +1015,12 @@ idle_changed_cb (GpmIdle    *idle,
 	case GPM_IDLE_MODE_SYSTEM:
 		gpm_debug ("Idle state changed: SYSTEM");
 
+		if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+			gpm_debug ("Skipping suppressed timeout action");
+			return;
+		}
 		/* can only be hibernate or suspend */
+		gpm_manager_set_reason (manager, "the system state is idle");
 		manager_policy_do (manager, GPM_PREF_SLEEP_TYPE);
 
 		break;
@@ -940,30 +1052,65 @@ dpms_mode_changed_cb (GpmDpms    *dpms,
 
 	gpm_debug ("emitting dpms-mode-changed : %s", gpm_dpms_mode_to_string (mode));
 	g_signal_emit (manager,
-		       signals [DPMS_MODE_CHANGED],
-		       0,
-		       gpm_dpms_mode_to_string (mode));
+			signals [DPMS_MODE_CHANGED],
+			0,
+			gpm_dpms_mode_to_string (mode));
+}
+
+static void
+battery_button_pressed (GpmManager *manager)
+{
+	char *message;
+
+	gpm_power_get_status_summary (manager->priv->power, &message, NULL);
+
+	gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+			      GPM_NOTIFY_TIMEOUT_LONG,
+			      _("Power Information"),
+			      NULL,
+			      message);
+	g_free (message);
 }
 
 static void
 power_button_pressed (GpmManager   *manager,
 		      gboolean	    state)
 {
-	gpm_debug ("power button changed: %d", state);
-
-	/* Log out interactively */
-	gnome_client_request_save (gnome_master_client (),
-				   GNOME_SAVE_GLOBAL,
-				   TRUE, GNOME_INTERACT_ANY, FALSE,  TRUE);
+	if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+		gpm_debug ("Skipping suppressed power button press");
+		return;
+	}
+	gpm_debug ("power button pressed");
+	gpm_manager_set_reason (manager, "the power button has been pressed");
+	manager_policy_do (manager, GPM_PREF_BUTTON_POWER);
 }
 
 static void
 suspend_button_pressed (GpmManager   *manager,
 			gboolean      state)
 {
+	if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+		gpm_debug ("Skipping suppressed suspend button press");
+		return;
+	}
+	gpm_debug ("suspend button pressed");
+	gpm_manager_set_reason (manager, "the suspend button has been pressed");
 	manager_policy_do (manager, GPM_PREF_BUTTON_SUSPEND);
 }
 
+static void
+hibernate_button_pressed (GpmManager   *manager,
+			gboolean      state)
+{
+	if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+		gpm_debug ("Skipping suppressed hibernate button press");
+		return;
+	}
+	gpm_debug ("hibernate button pressed");
+	gpm_manager_set_reason (manager, "the hibernate button has been pressed");
+	manager_policy_do (manager, GPM_PREF_BUTTON_HIBERNATE);
+}
+
 static void
 lid_button_pressed (GpmManager	 *manager,
 		    gboolean	  state)
@@ -972,7 +1119,7 @@ lid_button_pressed (GpmManager	 *manager,
 
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
-	gpm_debug ("button changed: %d", state);
+	gpm_debug ("lid button changed: %d", state);
 
 	/* We keep track of the lid state so we can do the 
 	   lid close on battery action if the ac_adapter is removed when the laptop
@@ -980,14 +1127,23 @@ lid_button_pressed (GpmManager	 *manager,
 	manager->priv->lid_is_closed = state;
 
 	if (state) {
+		/* Disable the screensaver, as we don't want this starting
+		   when the lid is shut */
+		gpm_screensaver_inhibit_activation ("gnome-power-manager has "
+						    "detected that the lid is closed");
 		if (on_ac) {
 			gpm_debug ("Performing AC policy");
+			gpm_manager_set_reason (manager, "the lid has been closed on ac power");
 			manager_policy_do (manager, GPM_PREF_AC_BUTTON_LID);
 		} else {
 			gpm_debug ("Performing battery policy");
+			gpm_manager_set_reason (manager, "the lid has been closed on battery power");
 			manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
 		}
 	} else {
+		/* re-enable the screensaver */
+		gpm_screensaver_allow_activation ();
+
 		/* we turn the lid dpms back on unconditionally */
 		gpm_manager_unblank_screen (manager, NULL);
 	}
@@ -996,42 +1152,42 @@ lid_button_pressed (GpmManager	 *manager,
 static void
 power_button_pressed_cb (GpmPower   *power,
 			 const char *type,
-			 const char *details,
 			 gboolean    state,
 			 GpmManager *manager)
 {
-	gpm_debug ("Received a button press event type=%s details=%s state=%d",
-		 type, details, state);
+	gpm_debug ("Button press event type=%s state=%d", type, state);
 
-	if (strcmp (type, "power") == 0) {
+	/* simulate user input */
+	gpm_screensaver_poke ();
+
+	if (strcmp (type, GPM_BUTTON_POWER) == 0) {
 		power_button_pressed (manager, state);
 
-	} else if (strcmp (type, "sleep") == 0) {
+	} else if (strcmp (type, GPM_BUTTON_SLEEP) == 0) {
 		suspend_button_pressed (manager, state);
 
-	} else if (strcmp (type, "lid") == 0) {
+	} else if (strcmp (type, GPM_BUTTON_SUSPEND) == 0) {
+		suspend_button_pressed (manager, state);
+
+	} else if (strcmp (type, GPM_BUTTON_HIBERNATE) == 0) {
+		hibernate_button_pressed (manager, state);
+
+	} else if (strcmp (type, GPM_BUTTON_LID) == 0) {
 		lid_button_pressed (manager, state);
 
-	} else if (strcmp (type, "virtual") == 0) {
-		if (details == NULL) {
-			return;
-		}
+	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_UP) == 0) ||
+		   (strcmp (type, GPM_BUTTON_BRIGHT_UP_DEP) == 0)) {
+		gpm_brightness_level_up (manager->priv->brightness);
 
-		if (strcmp (details, "BrightnessUp") == 0) {
-			gpm_brightness_level_up (manager->priv->brightness);
+	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_DOWN) == 0) ||
+		   (strcmp (type, GPM_BUTTON_BRIGHT_DOWN_DEP) == 0)) {
+		gpm_brightness_level_down (manager->priv->brightness);
 
-		} else if (strcmp (details, "BrightnessDown") == 0) {
-			gpm_brightness_level_down (manager->priv->brightness);
+	} else if (strcmp (type, GPM_BUTTON_LOCK) == 0) {
+		gpm_screensaver_lock ();
 
-		} else if (strcmp (details, "Suspend") == 0) {
-			gpm_manager_suspend (manager, NULL);
-
-		} else if (strcmp (details, "Hibernate") == 0) {
-			gpm_manager_hibernate (manager, NULL);
-
-		} else if (strcmp (details, "Lock") == 0) {
-			gpm_screensaver_lock ();
-		}
+	} else if (strcmp (type, GPM_BUTTON_BATTERY) == 0) {
+		battery_button_pressed (manager);
 	}
 }
 
@@ -1042,6 +1198,9 @@ power_on_ac_changed_cb (GpmPower   *power,
 {
 	gpm_debug ("Setting on-ac: %d", on_ac);
 
+	/* simulate user input, to fix #333525 */
+	gpm_screensaver_poke ();
+
 	/* If we are on AC power we should show warnings again */
 	if (on_ac) {
 		gpm_debug ("Resetting last_primary_warning to NONE");
@@ -1060,6 +1219,7 @@ power_on_ac_changed_cb (GpmPower   *power,
 	   when the laptop is closed and on battery. Fixes #331655 */
 	if (! on_ac && manager->priv->lid_is_closed) {
 		gpm_debug ("Doing battery policy when lid closed and power removed");
+		gpm_manager_set_reason (manager, "the lid has been closed, and the ac adapter removed");
 		manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
 	}
 }
@@ -1109,7 +1269,8 @@ battery_low_get_title (GpmWarning warning_type)
 {
 	char *title = NULL;
 
-	if (warning_type == GPM_WARNING_CRITICAL) {
+	if (warning_type == GPM_WARNING_ACTION ||
+	    warning_type == GPM_WARNING_CRITICAL) {
 
 		title = _("Power Critically Low");
 
@@ -1140,13 +1301,13 @@ manager_critical_action_do (GpmManager *manager)
 /* performs critical action is required, and displays notifications */
 static void
 battery_status_changed_primary (GpmManager	      *manager,
-			        GpmPowerBatteryKind    battery_kind,
+				 GpmPowerBatteryKind    battery_kind,
 				GpmPowerBatteryStatus *battery_status)
 {
 	GpmWarning  warning_type;
 	gboolean    show_notify;
-	char       *message = NULL;
-	char       *remaining = NULL;
+	char	*message = NULL;
+	char	*remaining = NULL;
 	const char *title = NULL;
 	gboolean    on_ac;
 
@@ -1168,18 +1329,14 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 		if (show_notify) {
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-					      5000,
+					      GPM_NOTIFY_TIMEOUT_SHORT,
 					      _("Battery Charged"),
 					      NULL,
 					      _("Your battery is now fully charged"));
 		}
-		return;
 	}
 
-	if (battery_status->percentage_charge != manager->priv->last_primary_percentage_change) {
-		/* should we fire an event or something? */
-		manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
-	}
+	manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
 
 	if (! battery_status->is_discharging) {
 		gpm_debug ("%s is not discharging", battery_kind_to_string (battery_kind));
@@ -1203,6 +1360,11 @@ battery_status_changed_primary (GpmManager	      *manager,
 		const char *warning = NULL;
 		const char *action;
 
+		if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+			gpm_debug ("Skipping suppressed critical action");
+			return;
+		}
+
 		/* we have to do different warnings depending on the policy */
 		action = gconf_client_get_string (manager->priv->gconf_client,
 						  GPM_PREF_BATTERY_CRITICAL, NULL);
@@ -1227,19 +1389,21 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 		if (warning) {
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-					      5000,
+					      GPM_NOTIFY_TIMEOUT_LONG,
 					      _("Critical action"),
 					      NULL,
 					      warning);
 		}
+		gpm_manager_set_reason (manager, "we are critically low for the primary battery");
 		/* wait 10 seconds for user-panic */
 		g_timeout_add (1000*10, (GSourceFunc) manager_critical_action_do, manager);
 	}
 
 	/* Always check if we already notified the user */
 	if (warning_type > manager->priv->last_primary_warning) {
+		int timeout;
+
 		manager->priv->last_primary_warning = warning_type;
-		title = battery_low_get_title (warning_type);
 		remaining = gpm_get_timestring (battery_status->remaining_time);
 
 		/* Do different warnings for each GPM_WARNING */
@@ -1249,19 +1413,26 @@ battery_status_changed_primary (GpmManager	      *manager,
 							     GPM_PREF_NOTIFY_ACADAPTER, NULL);
 			if (show_notify) {
 				message = g_strdup_printf (_("The AC Power has been unplugged. "
-						             "The system is now using battery power."));
+							      "The system is now using battery power."));
+				timeout = GPM_NOTIFY_TIMEOUT_SHORT;
 			}
 		} else {
 			message = g_strdup_printf (_("You have approximately <b>%s</b> "
 						     "of remaining battery life (%d%%). "
 						     "Plug in your AC Adapter to avoid losing data."),
 						   remaining, battery_status->percentage_charge);
+				timeout = GPM_NOTIFY_TIMEOUT_LONG;
+		}
+		if (message) {
+			title = battery_low_get_title (warning_type);
+			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+					      timeout,
+					      title,
+					      NULL,
+					      message);
+			g_free (message);
 		}
-		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000, title, NULL, message);
-
 		g_free (remaining);
-		g_free (message);
 	} else {
 		gpm_debug ("Already notified %i", warning_type);
 	}
@@ -1269,7 +1440,7 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 /* displays notifications */
 static void
-battery_status_changed_ups (GpmManager	          *manager,
+battery_status_changed_ups (GpmManager		   *manager,
 			    GpmPowerBatteryKind	   battery_kind,
 			    GpmPowerBatteryStatus *battery_status)
 {
@@ -1320,7 +1491,10 @@ battery_status_changed_ups (GpmManager	          *manager,
 						   remaining, battery_status->percentage_charge);
 		}
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000, title, NULL, message);
+				      GPM_NOTIFY_TIMEOUT_LONG,
+				      title,
+				      NULL,
+				      message);
 
 		g_free (remaining);
 		g_free (message);
@@ -1387,15 +1561,18 @@ battery_status_changed_misc (GpmManager	    	   *manager,
 				   name, battery_status->percentage_charge);
 
 	gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-			      5000, title, NULL, message);
+			      GPM_NOTIFY_TIMEOUT_LONG,
+			      title,
+			      NULL,
+			      message);
 
 	g_free (message);
 }
 
 static void
-power_battery_status_changed_cb (GpmPower	       *power,
+power_battery_status_changed_cb (GpmPower		*power,
 				 GpmPowerBatteryKind	battery_kind,
-				 GpmManager	       *manager)
+				 GpmManager		*manager)
 {
 	GpmPowerBatteryStatus battery_status;
 
@@ -1495,6 +1672,7 @@ callback_gconf_key_changed (GConfClient *client,
 	gint	    brightness;
 	GpmManager *manager = GPM_MANAGER (user_data);
 	gboolean    on_ac;
+	gboolean enabled;
 
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
@@ -1548,6 +1726,18 @@ callback_gconf_key_changed (GConfClient *client,
 			gpm_brightness_level_set (manager->priv->brightness, brightness);
 		}
 
+	} else if (strcmp (entry->key, GPM_PREF_CAN_SUSPEND) == 0) {
+		gpm_manager_can_suspend (manager, &enabled, NULL);
+		gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
+
+	} else if (strcmp (entry->key, GPM_PREF_CAN_HIBERNATE) == 0) {
+		gpm_manager_can_hibernate (manager, &enabled, NULL);
+		gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
+
+	} else if (strcmp (entry->key, GPM_PREF_POLICY_TIMEOUT) == 0) {
+		manager->priv->suppress_policy_timeout =
+			gconf_client_get_int (manager->priv->gconf_client,
+				  	      GPM_PREF_POLICY_TIMEOUT, NULL);
 	}
 }
 
@@ -1556,15 +1746,25 @@ static void
 gpm_manager_tray_icon_hibernate (GpmManager   *manager,
 				 GpmTrayIcon  *tray)
 {
+	if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+		gpm_debug ("Skipping suppressed hibernate signal");
+		return;
+	}
 	gpm_debug ("Received hibernate signal from tray icon");
+	gpm_manager_set_reason (manager, "user clicked hibernate from tray menu");
 	gpm_manager_hibernate (manager, NULL);
 }
 
 static void
 gpm_manager_tray_icon_suspend (GpmManager   *manager,
-			       GpmTrayIcon  *tray)
+				GpmTrayIcon  *tray)
 {
+	if (gpm_manager_is_policy_timout_valid (manager) == FALSE) {
+		gpm_debug ("Skipping suppressed suspend signal");
+		return;
+	}
 	gpm_debug ("Received supend signal from tray icon");
+	gpm_manager_set_reason (manager, "user clicked suspend from tray menu");
 	gpm_manager_suspend (manager, NULL);
 }
 #endif
@@ -1579,6 +1779,14 @@ hal_battery_removed_cb (GpmHalMonitor *monitor,
 	tray_icon_update (manager);
 }
 
+static void
+gpm_manager_tray_icon_show_info (GpmManager   *manager,
+				  GpmTrayIcon  *tray)
+{
+	gpm_debug ("Received show-info signal from tray icon");
+	gpm_info_show_window (manager->priv->info);
+}
+
 static void
 gpm_manager_init (GpmManager *manager)
 {
@@ -1636,6 +1844,11 @@ gpm_manager_init (GpmManager *manager)
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
 
+	gpm_debug ("initialising info infrastructure");
+	manager->priv->info = gpm_info_new ();
+	/* logging system needs access to the power stuff... bit of a bodge */
+	gpm_info_set_power (manager->priv->info, manager->priv->power);
+
 #if ACTIONS_MENU_ENABLED
 	gpm_manager_can_suspend (manager, &enabled, NULL);
 	gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
@@ -1653,6 +1866,11 @@ gpm_manager_init (GpmManager *manager)
 				 manager,
 				 G_CONNECT_SWAPPED);
 #endif
+	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
+				 "show-info",
+				 G_CALLBACK (gpm_manager_tray_icon_show_info),
+				 manager,
+				 G_CONNECT_SWAPPED);
 
 	/* coldplug so we are in the correct state at startup */
 	sync_dpms_policy (manager);
@@ -1676,16 +1894,25 @@ gpm_manager_init (GpmManager *manager)
 		manager->priv->last_primary_warning = GPM_WARNING_DISCHARGING;
 	}
 
+	/* we set the reason to be unknown */
+	gpm_manager_set_reason (manager, "of an unknown reason");
+
 	/* We can disable this if the ACPI BIOS is fucked, and the
 	   time_remaining is therefore inaccurate or just plain wrong. */
 	use_time = gconf_client_get_bool (manager->priv->gconf_client,
 					  GPM_PREF_USE_TIME_POLICY, NULL);
+	manager->priv->use_time_to_notify = use_time;
 	if (use_time) {
 		gpm_debug ("Using per-time notification policy");
 	} else {
 		gpm_debug ("Using percentage notification policy");
 	}
-	manager->priv->use_time_to_notify = use_time;
+	
+	manager->priv->suppress_policy_timeout =
+		gconf_client_get_int (manager->priv->gconf_client,
+				      GPM_PREF_POLICY_TIMEOUT, NULL);
+	gpm_debug ("Using a supressed policy timeout of %i seconds",
+		   manager->priv->suppress_policy_timeout);
 }
 
 static void
@@ -1712,6 +1939,10 @@ gpm_manager_finalize (GObject *object)
 		g_object_unref (manager->priv->idle);
 	}
 
+	if (manager->priv->info != NULL) {
+		g_object_unref (manager->priv->info);
+	}
+
 	if (manager->priv->power != NULL) {
 		g_object_unref (manager->priv->power);
 	}
