@@ -91,9 +91,9 @@ gpm_screensaver_is_running (void)
 	if (!gpm_screensaver_get_session_conn (&session_connection))
 		return FALSE;
 	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
-			GS_LISTENER_SERVICE,
-			GS_LISTENER_PATH,
-			GS_LISTENER_INTERFACE);
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
 	if (!dbus_g_proxy_call (gs_proxy, "getActive", &error,
 				G_TYPE_INVALID,
 				G_TYPE_BOOLEAN, &temp, G_TYPE_INVALID)) {
@@ -124,9 +124,9 @@ gpm_screensaver_enable_throttle (gboolean enable)
 	if (!gpm_screensaver_get_session_conn (&session_connection))
 		return FALSE;
 	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
-			GS_LISTENER_SERVICE,
-			GS_LISTENER_PATH,
-			GS_LISTENER_INTERFACE);
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
 	if (!dbus_g_proxy_call (gs_proxy, "setThrottleEnabled", &error,
 				G_TYPE_BOOLEAN, enable, G_TYPE_INVALID,
 				G_TYPE_INVALID)) {
@@ -154,15 +154,90 @@ gpm_screensaver_lock (void)
 {
 	DBusGConnection *session_connection = NULL;
 	DBusGProxy *gs_proxy = NULL;
-	gpm_debug ("lock");
+	int sleepcount = 0;
+
+	if (! gpm_screensaver_is_running ()) {
+		gpm_debug ("Not locking, as gnome-screensaver not running");
+		return FALSE;
+	}
+
+	gpm_debug ("doing gnome-screensaver lock");
+
 	if (!gpm_screensaver_get_session_conn (&session_connection))
 		return FALSE;
 	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
-			GS_LISTENER_SERVICE,
-			GS_LISTENER_PATH,
-			GS_LISTENER_INTERFACE);
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
 	dbus_g_proxy_call_no_reply (gs_proxy, "Lock", G_TYPE_INVALID);
 	g_object_unref (G_OBJECT (gs_proxy));
+
+	/* When we send the Lock signal to g-ss it takes maybe a second
+	   or so to fade the screen and lock. If we suspend mid fade then on
+	   resume the X display is still present for a split second
+	   (since fade is gamma) and as such it can leak information.
+	   Instead we wait until g-ss reports running and thus blanked
+	   solidly before we continue from the screensaver_lock action.
+	   The interior of g-ss is async, so we cannot get the dbus method
+	   to block until lock is complete. */
+	while (! gpm_screensaver_is_running ()) {
+		/* Sleep for 1/10s */
+		g_usleep (1000 * 100);
+		if (sleepcount++ > 50) {
+			break;
+		}
+	}
+
+	return TRUE;
+}
+
+/** Prevent the screensaver from activating
+ *
+ *  @param			The reason, e.g. "because lid is closed"
+ *  @return			TRUE if gnome-screensaver locked the screen.
+ */
+gboolean
+gpm_screensaver_inhibit_activation (const char *reason)
+{
+	DBusGConnection *session_connection = NULL;
+	DBusGProxy *gs_proxy = NULL;
+
+	gpm_debug ("inhibit activation");
+
+	if (!gpm_screensaver_get_session_conn (&session_connection))
+		return FALSE;
+	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
+	dbus_g_proxy_call_no_reply (gs_proxy, "InhibitActivation",
+				    G_TYPE_STRING, reason,
+				    G_TYPE_INVALID);
+	g_object_unref (G_OBJECT (gs_proxy));
+	return TRUE;
+}
+
+/** Allow the screensaver to activate
+ *
+ *  @return			TRUE if gnome-screensaver locked the screen.
+ */
+gboolean
+gpm_screensaver_allow_activation (void)
+{
+	DBusGConnection *session_connection = NULL;
+	DBusGProxy *gs_proxy = NULL;
+
+	gpm_debug ("allow activation");
+
+	if (!gpm_screensaver_get_session_conn (&session_connection))
+		return FALSE;
+	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
+	dbus_g_proxy_call_no_reply (gs_proxy, "AllowActivation",
+				    G_TYPE_INVALID);
+	g_object_unref (G_OBJECT (gs_proxy));
 	return TRUE;
 }
 
@@ -180,9 +255,9 @@ gpm_screensaver_poke (void)
 	if (!gpm_screensaver_get_session_conn (&session_connection))
 		return FALSE;
 	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
-			GS_LISTENER_SERVICE,
-			GS_LISTENER_PATH,
-			GS_LISTENER_INTERFACE);
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
 	dbus_g_proxy_call_no_reply (gs_proxy, "Poke", G_TYPE_INVALID);
 	g_object_unref (G_OBJECT (gs_proxy));
 	return TRUE;
@@ -204,9 +279,9 @@ gpm_screensaver_get_idle (gint *time)
 	if (!gpm_screensaver_get_session_conn (&session_connection))
 		return FALSE;
 	gs_proxy = dbus_g_proxy_new_for_name (session_connection,
-			GS_LISTENER_SERVICE,
-			GS_LISTENER_PATH,
-			GS_LISTENER_INTERFACE);
+					      GS_LISTENER_SERVICE,
+					      GS_LISTENER_PATH,
+					      GS_LISTENER_INTERFACE);
 	if (!dbus_g_proxy_call (gs_proxy, "getActiveTime", &error,
 				G_TYPE_INVALID,
 				G_TYPE_UINT, time, G_TYPE_INVALID)) {
