@@ -49,41 +49,38 @@ gpm_stock_icons_init (void)
 
 	static const GpmStockIcon items [] = {
 		/* GPM custom icons */
-		{ GPM_STOCK_AC_0_OF_8, TRUE },
-		{ GPM_STOCK_AC_1_OF_8, TRUE },
-		{ GPM_STOCK_AC_2_OF_8, TRUE },
-		{ GPM_STOCK_AC_3_OF_8, TRUE },
-		{ GPM_STOCK_AC_4_OF_8, TRUE },
-		{ GPM_STOCK_AC_5_OF_8, TRUE },
-		{ GPM_STOCK_AC_6_OF_8, TRUE },
-		{ GPM_STOCK_AC_7_OF_8, TRUE },
-		{ GPM_STOCK_AC_8_OF_8, TRUE },
-		{ GPM_STOCK_BAT_0_OF_8, TRUE },
-		{ GPM_STOCK_BAT_1_OF_8, TRUE },
-		{ GPM_STOCK_BAT_2_OF_8, TRUE },
-		{ GPM_STOCK_BAT_3_OF_8, TRUE },
-		{ GPM_STOCK_BAT_4_OF_8, TRUE },
-		{ GPM_STOCK_BAT_5_OF_8, TRUE },
-		{ GPM_STOCK_BAT_6_OF_8, TRUE },
-		{ GPM_STOCK_BAT_7_OF_8, TRUE },
-		{ GPM_STOCK_BAT_8_OF_8, TRUE },
-		{ GPM_STOCK_UPS_0_OF_8, TRUE },
-		{ GPM_STOCK_UPS_1_OF_8, TRUE },
-		{ GPM_STOCK_UPS_2_OF_8, TRUE },
-		{ GPM_STOCK_UPS_3_OF_8, TRUE },
-		{ GPM_STOCK_UPS_4_OF_8, TRUE },
-		{ GPM_STOCK_UPS_5_OF_8, TRUE },
-		{ GPM_STOCK_UPS_6_OF_8, TRUE },
-		{ GPM_STOCK_UPS_7_OF_8, TRUE },
-		{ GPM_STOCK_UPS_8_OF_8, TRUE },
 		{ GPM_STOCK_AC_ADAPTER, TRUE },
-		{ GPM_STOCK_MOUSE, TRUE },
-		{ GPM_STOCK_KEYBOARD, TRUE },
-		{ GPM_STOCK_AC_CHARGED, TRUE },
-
-		/* gnome-icon-theme icons */
-		{ GNOME_DEV_MEMORY, FALSE },
-		{ GNOME_DEV_HARDDISK, FALSE },
+		{ GPM_STOCK_KEYBOARD_LOW, TRUE },
+		{ GPM_STOCK_MOUSE_LOW, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGED, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGING_000, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGING_020, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGING_040, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGING_060, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGING_080, TRUE },
+		{ GPM_STOCK_BATTERY_CHARGING_100, TRUE },
+		{ GPM_STOCK_BATTERY_DISCHARGING_000, TRUE },
+		{ GPM_STOCK_BATTERY_DISCHARGING_020, TRUE },
+		{ GPM_STOCK_BATTERY_DISCHARGING_040, TRUE },
+		{ GPM_STOCK_BATTERY_DISCHARGING_060, TRUE },
+		{ GPM_STOCK_BATTERY_DISCHARGING_080, TRUE },
+		{ GPM_STOCK_BATTERY_DISCHARGING_100, TRUE },
+		{ GPM_STOCK_UPS_CHARGED, TRUE },
+		{ GPM_STOCK_UPS_CHARGING_000, TRUE },
+		{ GPM_STOCK_UPS_CHARGING_020, TRUE },
+		{ GPM_STOCK_UPS_CHARGING_040, TRUE },
+		{ GPM_STOCK_UPS_CHARGING_060, TRUE },
+		{ GPM_STOCK_UPS_CHARGING_080, TRUE },
+		{ GPM_STOCK_UPS_CHARGING_100, TRUE },
+		{ GPM_STOCK_UPS_DISCHARGING_000, TRUE },
+		{ GPM_STOCK_UPS_DISCHARGING_020, TRUE },
+		{ GPM_STOCK_UPS_DISCHARGING_040, TRUE },
+		{ GPM_STOCK_UPS_DISCHARGING_060, TRUE },
+		{ GPM_STOCK_UPS_DISCHARGING_080, TRUE },
+		{ GPM_STOCK_UPS_DISCHARGING_100, TRUE },
+		{ GPM_STOCK_POWER_INFORMATION, TRUE },
+		{ GPM_STOCK_SUSPEND_TO_DISK, TRUE },
+		{ GPM_STOCK_SUSPEND_TO_RAM, TRUE },
 
 	};
 
