@@ -81,6 +81,7 @@ struct GpmTrayIconPrivate
 enum {
 	SUSPEND,
 	HIBERNATE,
+	SHOW_INFO,
 	LAST_SIGNAL
 };
 
@@ -95,20 +96,23 @@ typedef enum {
 	LIBNOTIFY_URGENCY_LOW      = 3
 } LibNotifyEventType;
 
-static void gpm_tray_icon_suspend_cb          (GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_hibernate_cb        (GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_preferences_cb (GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_help_cb        (GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_about_cb       (GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_suspend_cb		(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_hibernate_cb		(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_info_cb	 	(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_preferences_cb	(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_help_cb		(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_about_cb		(GtkAction *action, GpmTrayIcon *icon);
 
 static GtkActionEntry gpm_tray_icon_action_entries [] =
 {
-	{ "TraySuspend", GNOME_DEV_MEMORY, N_("_Suspend"),
+	{ "TraySuspend", GPM_STOCK_SUSPEND_TO_RAM, N_("_Suspend"),
 	  NULL, N_("Suspend the computer"), G_CALLBACK (gpm_tray_icon_suspend_cb) },
-	{ "TrayHibernate", GNOME_DEV_HARDDISK, N_("Hi_bernate"),
+	{ "TrayHibernate", GPM_STOCK_SUSPEND_TO_DISK, N_("Hi_bernate"),
 	  NULL, N_("Make the computer go to sleep"), G_CALLBACK (gpm_tray_icon_hibernate_cb) },
 	{ "TrayPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"),
 	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_preferences_cb) },
+	{ "TrayInfo", GPM_STOCK_POWER_INFORMATION, N_("_Information"),
+	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_info_cb) },
 	{ "TrayHelp", GTK_STOCK_HELP, N_("_Help"), NULL,
 	  NULL, G_CALLBACK (gpm_tray_icon_show_help_cb) },
 	{ "TrayAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
@@ -192,6 +196,14 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 	}
 }
 
+static void
+gpm_tray_icon_show_info_cb (GtkAction   *action,
+			    GpmTrayIcon *icon)
+{
+	gpm_debug ("emitting show_info");
+	g_signal_emit (icon, signals [SHOW_INFO], 0);
+}
+
 static void
 gpm_tray_icon_hibernate_cb (GtkAction   *action,
 			    GpmTrayIcon *icon)
@@ -278,6 +290,8 @@ gpm_tray_icon_show_about_cb (GtkAction  *action,
 	license_trans = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n",
 				     _(license[2]), "\n\n", _(license[3]), "\n",  NULL);
 
+	gtk_window_set_default_icon_from_file (GPM_DATA "gnome-power-manager.png", NULL);
+
 	gtk_show_about_dialog (NULL,
 		               "name", GPM_NAME,
 		               "version", VERSION,
@@ -436,6 +450,9 @@ gpm_tray_icon_constructor (GType                  type,
 					   "    <menuitem action=\"TrayHibernate\" />"
 					   "    <separator />"
 					   "    <menuitem action=\"TrayPreferences\" />"
+#if EXPERIMENTAL_FEATURES_ENABLED
+					   "    <menuitem action=\"TrayInfo\" />"
+#endif
 					   "    <separator />"
 					   "    <menuitem action=\"TrayHelp\" />"
 					   "    <menuitem action=\"TrayAbout\" />"
@@ -492,7 +509,16 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE,
 		              0);
-
+	signals [SHOW_INFO] =
+		g_signal_new ("show-info",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
+		              NULL,
+		              NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+		              0);
 }
 
 void
@@ -504,6 +530,7 @@ gpm_tray_icon_show (GpmTrayIcon *icon,
 		icon->priv->is_visible = TRUE;
 	} else {
 		gtk_widget_hide_all (GTK_WIDGET (icon));
+		gtk_widget_unrealize (GTK_WIDGET (icon));
 		icon->priv->is_visible = FALSE;
 	}
 }
@@ -605,7 +632,7 @@ notification_closed_cb (NotifyNotification *notify,
 
 static gboolean
 libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,
+		 guint                    timeout,	/* in seconds */
 		 const char              *subject,
 		 const char              *content,
 		 const LibNotifyEventType urgency)
@@ -617,9 +644,11 @@ libnotify_event (GpmTrayIcon             *tray,
 		notify_notification_close (tray->priv->notify, NULL);
 	}
 
-	tray->priv->notify = notify_notification_new (subject, content, GNOME_DEV_BATTERY, NULL);
+	tray->priv->notify = notify_notification_new (subject, content,
+						      GPM_STOCK_BATTERY_DISCHARGING_100,
+						      NULL);
 
-	notify_notification_set_timeout (tray->priv->notify, timeout);
+	notify_notification_set_timeout (tray->priv->notify, timeout * 1000);
 
 	if (tray->priv->is_visible) {
 		get_widget_position (GTK_WIDGET (tray), &x, &y);
@@ -647,7 +676,7 @@ libnotify_event (GpmTrayIcon             *tray,
 
 static gboolean
 libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,
+		 guint                    timeout,	/* in seconds */
 		 const char              *subject,
 		 const char              *content,
 		 const LibNotifyEventType urgency)
@@ -687,7 +716,7 @@ libnotify_event (GpmTrayIcon             *tray,
 						       content,
 						       icon, /* icon */
 						       TRUE,
-						       timeout / 1000,
+						       timeout,
 						       hints, /* hints */
 						       NULL, /* no user data */
 						       0);   /* no actions */
@@ -705,7 +734,7 @@ libnotify_event (GpmTrayIcon             *tray,
 
 static gboolean
 libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,
+		 guint                    timeout,	/* in seconds */
 		 const char              *subject,
 		 const char              *content,
 		 const LibNotifyEventType urgency)
@@ -745,7 +774,7 @@ libnotify_event (GpmTrayIcon             *tray,
 
 void
 gpm_tray_icon_notify (GpmTrayIcon *icon,
-		      guint        timeout,
+		      guint        timeout,	/* in seconds */
 		      const char  *primary,
 		      GtkWidget   *msgicon,
 		      const char  *secondary)
